.TH XTMENU 1 "October 7, 1991" "Version 1.1"
.SH "NAME"
xtmenu \- An X windows menu program
.SH "SYNTAX"
.B xtmenu
[ -help ] [ -print_help ] [ -m menufile ] [ -stdin ]
[ -noquit ] [ -horizontal ] [ -tvtwm ] [ -sep <char> ] [ -sub <char> ]
[ +/- keysave ] [ +/- highlight]
.SH "DESCRIPTION"
.I Xtmenu
allows a user to create menus.
Menu items may be inserted,
deleted, or modified on-the-fly.
A menu may also execute commands in other
.I xterm
windows.
.PP
.SH "OPTIONS"
Here are the
.I xtmenu
command line options:
.nf
.ta 0.5i +1.0i
	-help	Popup Help Menu (Resource *help)
	-print_help	Print Help to stdout (Resource *printHelp)
	-m <menufile>	Read Menu from File (Resource *menu)
	-stdin	Read Menu from Standard Input (Resource *std_in)
	-noquit	Don't quit after first button press (Resource *noquit)
		(Default quits after first button press.)
	-horizontal	Pop-up Menu Horizontally (Resource *horizontal)
		(Default is a vertical menu.)
	-tvtwm	Using Virtual Desktop Features (Resource *tvtwm)
		(Default is no virtual root)
	-sep <char>	Multiple command separator character (Resource *sep)
		(Default separator character of '#')
	-sub <char>	Environment variable substitution character (Resource *sub)
		(Default substitution character of '@')
	-keysave	Turn Off Key Saves at Startup (Resource *keySave)
	+keysave	Turn On Key Saves at Startup (Resource *keySave)
		(Default is Key Saves On at Startup)
	-highlight	Do Not Highlight Selected Item (Resource *highlight)
	+highlight	Highlight Selected Item (Resource *highlight)
		(Default is Highlighting On)
.fi
.PP
.SH "Menu File Format"
A
.I menufile
is a text file where each line is a menu item.
A menu item consists of a label
and a command associated with the label.
Comments may be inserted between menu items
by beginning the line with a '#' (pound symbol).
A menu item may be continued over several lines by ending each line
with a '\\' (backslash).
.PP
.nf
.ta 0.5i +2.0i
	# Sample Menu File
	"Label 1"	Command 1
	"Label 2"	Command 2
	"Blank Command"
	"Menu Command"	!menu_command
	"System Command"	%system_command
	"Select Command"	>select_command
	"Text Window"		}text
	"File Window"		]file
	"Multiple Commands"	cmd1 # cmd2 # cmd3 ...
	"Environment Substitution"	@command
	# End of Sample Menu
.fi
.PP
Commands by default print their values unless a 'key'
appears at the beginning of the command.
Therefore, "Label 2" would print the text "Command 2" when executed.
A menu item with no command returns the menu
item number.  For example, "Blank Command" would print the value '3'.
.PP
Multiple commands per menu item may be separated with the '#' (pound)
separation character.
See 'Multiple Commands' below for more details.
.PP
Environment variable substitution is used
to execute any menu command with parameters.
Environment variables may be substituted before command execution
by prepending an '@' symbol to the command.
See 'Environment Variable Substitution' below for details.
.PP
.SH "Command Keys"
The key '!' signifies the command is a special Menu Command.
That is, '!exit' says when the user executes this menu item, the
.I xtmenu
program should exit.
.PP
A key of '%' indicates this is a System Command. "%ls"
means execute the 'ls' system command.
.PP
A key of '>' means execute this command in the current target window.
For example '>raise' means raise the target window.
The Select Command '>ls\\n' says execute 'ls' followed by a return sequence
in the target window
('\\n' indicates a return in The C Programming Language).
See 'Target Window' below for details on defining the target window.
.PP
The key '}' is used for creating simple popup text windows.  The
.I xtmenu
command '}hello' when executed would pop up a window containing the
text 'hello'.
.PP
Finally the key ']' creates a popup text window containing the
specified
.I file
data.
.PP
.SH "Menu Commands"
Menu commands are signified by the '!' key:
.nf
.ta 0.5i +1.5i
	!exit		Exits
	!help		XtMenu Help
	!label		A Plain Label
	!insert		Insert a Menu Item
	!modify		Modify a Menu Item
	!select		Select a Target Window
	!delete		Delete a Menu Item
	!raise		Raise a Window
	!lower		Lower a Window
	!iconify	Iconify a Window
	!print_item	Print a Menu Item to stdout
	!print_menu	Print Entire Menu to stdout
	!write_menu	Write Menu to a File
	!read_menu	Read Menu from a File
	!setenv [var [val]]	Set an Environment Variable
.fi
.PP
.SH "System Commands (Examples)"
System commands begin with the '%' key:
.nf
.ta 0.5i
	%ls -alg
	%xtmenu -tvtwm -noquit -geometry +$X+$Y -m menu.csh &
.fi
.PP
.SH "System Commands (Environment Variables)"
Before executing a '%system_command', the following
shell environment variables are set:
.nf
.ta 0.5i +1.5i
	ITEM	Menu Label Text
	OCMD	Original Menu Command
	CMD	Actual Menu Command
	X	Pointer X Coordinate
	Y	Pointer Y Coordinate
.fi
.PP
.SH "Select Commands"
The following 'select' commands (>select_command) execute in the target window:
.nf
.ta 0.5i +1.5i
	>ls -alg\\n		Execute 'ls -alg' on Target Window
	>/usr/ucb/clear\\n	Clear Target Window
	>raise			Raise Target Window
	>lower			Lower Target Window
	>iconify		Iconify Target Window
	>deiconify		DeIconify Target Window
.fi
.PP
.SH "Multiple Commands"
When a multiple command menu item is executed, other menu items may not
be executed until the multi-command reaches the last command or until
a sub-command is cancelled.
.PP
For example, the multi-command '!modify # !setenv'
would create a modify popup waiting until the modify completes before
proceeding to continue with the setenv popup.  If the '!modify' is cancelled
the multi-command ends.  If the user selects 'Modify',
the '!setenv' command is executed.  While the modify popup is active, no
other menu commands may be executed.
.PP
Also, the separation character (default '#') may be changed using
the '-sep' command line option or '*sep' resource.
.PP
.SH "Environment Variable Substitution"
An environment variable may be
set using the '!setenv' command.
The '!setenv' command has 3 forms: '!setenv', '!setenv VARIABLE',
and '!setenv VARIABLE VALUE'.
For example '!setenv FILE' would pop
up a window requesting the value for the environment variable 'FILE'.
The command '!setenv FILE test' would set the FILE environment variable
to test without requesting user input.
.PP
If the VALUE is enclosed with backquotes (`VALUE`), the VALUE
is executed as a system command and the resulting value(s) are
stored into the VARIABLE.
For example, !setenv FILES `ls` would set the FILES environment variable
to the files in the startup directory.
New line characters within an executed system command (\\n) are translated
into spaces.
If a user wants an environment variable to be backquoted, place two
sets of backquotes around the VALUE (!setenv HELLO ``hello``).
.PP
Once an environment variable is set, a system command could use the
variable by prepending a '$' in front of the variable.  For example,
the
.I xtmenu
system command '%echo $FILE' would echo the FILE variable in
the startup window.
.PP
To substitute the value of an environment variable in any
.I xtmenu
command before execution, prepend the entire command with an '@' symbol.  The
command '@}File is set to $FILE' would create a simple popup text
window where '$FILE' is replaced by the current environment variable FILE.
Also the command '@>echo $FILE\\n' would substitute the value of the
FILE environment variable before sending the echo command to the target window.
.PP
Environment variables may be surrounded by '{' and '}'
characters (left and right curly braces) allowing
two environment variables to be concatenated (for example ${FILE}${SUFFIX}).
A '$' at the end of the line is not changed.  A '$$' is replaced by a
single '$'.  There is no quote checking so "@}File '$FILE'" would be
replaced by a popup containing "File 'file_environment_value'".
.PP
Finally, the substitution character (default '@')
may be changed using the '-sub' command line option or '*sub' resource.
.PP
.SH "Keyboard Translations"
.I Xtmenu
allows every key press to have any of these translations:
.PP
.nf
.ta 0.5i +1.5i
	key_press()		Any Key Press/Release Event
	exec_item(#)		Execute an Item (1-max_items)
	exit()			Exit (!exit)
	toggle_key_save()	Toggle Keyboard Save
	insert_item()		Insert Selected Item (!insert)
	modify_item()		Modify Selected Item (!modify)
	setenv_item([var[,val]])	Set Environment Variable (!setenv)
	delete_item()		Delete Menu Item (!delete)
	delete_last_item()	Delete Last Menu Item
	select_item()		Select a Window (!select)
	print_menu()		Print the menu (!print_menu)
	write_menu()		Write Menu to a File (!write_menu)
	read_menu()		Read Menu from a File (!read_menu)
	raise_selection()	Raise the Selected Window
	lower_selection()	Lower the Selected Window
	raise_menu()		Raise the Menu
	lower_menu()		Lower the Menu
	iconify_selection()	Iconify the Selected Window
	deiconify_selection()	DeIconify the Selected Window
	pointer_center()	Center the Pointer in the Menu
	pointer_up()		Move the Pointer Up 1 Item
	pointer_down()		Move the Pointer Down 1 Item
	exec_current_item()	Execute Menu Item Under Pointer
	sync_up()		Synchronize the X Server
.fi
.PP
.SH "Target Window"
Before executing a Select Command (>select_command), the
user must 'select' a target window.
The target window may be selected
using the menu command '!select' or the keyboard translation 'select_item()'.
.PP
When selecting the target window, the cursor will change to
a 3-button mouse with the middle button highlighted.  The
user should press the mouse button (possibly while holding a
control, shift or meta key) while above the desired target window.  The
mouse button/keyboard sequence must be the same as that
used to
.I transmit
selected data from one window to another.  This
is usually the mouse middle button (with no keys) but may vary
depending upon the window manager configuration.
.PP
.I Xtmenu
commands may be
executed in the target window as long as SendEvents are allowed
by the selected target window.  (In
.I "xterm"
\, the allowSendEvents resource must be enabled to allow
.I xtmenu
to send 'synthetic' events.  This may be accomplished by (1) setting
the *allowSendEvents resource to TRUE in the
.I "$HOME/.Xdefaults" file
or (2) setting the 'Allow
SendEvents' menu option.  In
.I xterm
a <ctrl><Mouse_Button_1> should
display the 'Main Options' menu so the 'Allow SendEvents' option
may be enabled.  'man xterm' for more details on setting the
allowSendEvents resource.)
.PP
Once a target window is selected, any menu command beginning
with the '>' key may be executed to the target window.
See 'Select Commands' above.
.PP
Also once a target window is selected, the key_press() events are passed
through to the target window.  After the user presses a <Return>
the 'phrase' up to the <Return> is stored to the
current menu.  Thus a running
history may be maintained for a given target window.
.PP
.SH "Example Menu File"
Here is a larger sample menu with a variety of commands:
.nf
.ta 0.5i +1.5i
	#
	# Menu.csh
	# William Taylor
	# April 18, 1991
	#
	Xterm		%xterm &
	ls		>ls\\n
	"ls -alg"	>ls -alg\\n
	pwd		>pwd\\n
	Clear		>/usr/ucb/clear\\n
	Item		%echo "$ITEM" ; echo "$OCMD" ; echo "$CMD"
	Selection	!select
	# "! Raise"	!raise
	"> Raise"	>raise
	# "! Lower"	!lower
	"> Lower"	>lower
	Delete		!delete
	Iconify		!iconify
	Insert		!insert
	Modify		!modify
	Label		!label
	Help		!help
	"Print Item"	!print_item
	"Print Menu"	!print_menu
	"NewMenu ->"	%xtmenu -noquit -geometry +$X+$Y -m menu.csh &
	#
	# Multiple commands separated with a '#' character:
	"EV? %echo $EV"	!setenv EV # %echo $EV
	#
	# Environment variable substitution using an '@' character:
	"EV? >echo $EV"	!setenv EV # @>echo $EV\\n
	#
	Quit		!exit
.fi
.PP
.SH "Example Keyboard Translation Table"
Here is a sample set of keyboard translations.
The system administrator may save them in
.I "/usr/lib/X11/app-defaults/XtMenu."
The user may define the translations in
.I "$HOME/.Xdefaults"
or
.I "$HOME/XtMenu:"
.nf
.ta 0.5i +2.0i
  *switchbox.translations:	#override\\n\\
	None<KeyPress>F1:	exec_item(1)\\n\\
	None<KeyPress>F2:	exec_item(2)\\n\\
	None<KeyPress>F3:	exec_item(3)\\n\\
	None<KeyPress>F4:	exec_item(4)\\n\\
	None<KeyPress>F5:	exec_item(5)\\n\\
	None<KeyPress>F6:	exec_item(6)\\n\\
	None<KeyPress>F7:	exec_item(7)\\n\\
	None<KeyPress>F8:	exec_item(8)\\n\\
	None<KeyPress>F9:	exec_item(9)\\n\\
	None<KeyPress>F10:	exec_item(10)\\n\\
	None<KeyPress>F11:	exec_item(11)\\n\\
	None<KeyPress>F12:	exec_item(12)\\n\\
	Shift<KeyPress>F1:	exec_item(13)\\n\\
	Shift<KeyPress>F2:	exec_item(14)\\n\\
	Shift<KeyPress>F3:	exec_item(15)\\n\\
	Shift<KeyPress>F4:	exec_item(16)\\n\\
	Shift<KeyPress>F5:	exec_item(17)\\n\\
	Shift<KeyPress>F6:	exec_item(18)\\n\\
	Shift<KeyPress>F7:	exec_item(19)\\n\\
	Shift<KeyPress>F8:	exec_item(20)\\n\\
	Shift<KeyPress>F9:	exec_item(21)\\n\\
	Shift<KeyPress>F10:	exec_item(22)\\n\\
	Shift<KeyPress>F11:	exec_item(23)\\n\\
	Shift<KeyPress>F12:	exec_item(24)\\n\\
	Ctrl<KeyPress>F1:	setenv_item()\\n\\
	Ctrl<KeyPress>F2:	setenv_item("HELLO")\\n\\
	Ctrl<KeyPress>F3:	setenv_item("HELLO","hello")\\n\\
	Ctrl<KeyPress>q:	exit()\\n\\
	Ctrl<KeyPress>i:	insert_item()\\n\\
	Ctrl<KeyPress>m:	modify_item()\\n\\
	Ctrl<KeyPress>r:	delete_item()\\n\\
	Ctrl<KeyPress>y:	delete_last_item()\\n\\
	Ctrl<KeyPress>t:	select_item()\\n\\
	Ctrl<KeyPress>p:	print_menu()\\n\\
	Ctrl<KeyPress>o:	toggle_key_save()\\n\\
	Ctrl<KeyPress>a:	deiconify_selection() raise_selection() raise_menu()\\n\\
	Ctrl<KeyPress>s:	iconify_selection()\\n\\
	Ctrl<KeyPress>j:	pointer_down()\\n\\
	Ctrl<KeyPress>k:	pointer_up()\\n\\
	Ctrl<KeyPress>l:	exec_current_item()\\n\\
 	Ctrl<KeyPress>w:	write_menu()\\n\\
 	Ctrl<KeyPress>e:	read_menu()\\n\\
	<KeyPress>:		key_press()\\n\\
	<KeyRelease>:		key_press()
.fi
.PP
.SH "Future Endeavors"
Implement using Motif and Open Look Widgets.
.PP
Use a scrollbar inside a menu for lots of menu items (> MAX_SWITCHES).
.PP
Move windows with a '!move' menu command.
.PP
Resize windows with a '!resize' menu command.
.PP
Destroy windows with the '!delete' menu command.  Currently only
menu items may be deleted.
.PP
Position menu with a '-corner' as either upper_left
(default), upper_right, lower_left, or lower_right.
.PP
Allow the '}text' or ']file' popup to be modified instead of just viewed.
.PP
Allow the insertion of menu items at the cursor pointer
location rather than the end of the menu.
.PP
Allow '!move' to move menu items around the menu.
.PP
View the entire menu in a popup.
.PP
Attach a menu to a window (-attach).
.PP
Sort the menu labels (-sort).
.PP
Check for spelling on passed through history commands (-spell).
.PP
Allow commands to contain special characters (such as XK_Escape).
.PP
Allow labels to contain special characters (such as XC_pirate).
.PP
.SH "Open Issues"
.PP
When multiple level menus are brought up, if one is destroyed
should the menus below be destroyed?
.PP
Wouldn't it be nice if one
.I xtmenu
could modify another
.I "xtmenu's"
items?
.PP
.SH "Interesting Features"
Multiple text windows, '}text',  may be popped up dynamically throughout
the run.
.PP
Commands may be executed in other windows.
.PP
Errors are placed in popup windows.
.PP
Popup menus are not part of this program but may be created
by executing
.I xtmenu
as another system command '%xtmenu &'.
.PP
If the selected target window is covered up or iconified, select
commands (>select_command) are still executed in the hidden
target window.
.PP
.SH "Uses"
Several menus may be brought up where each menu is associated with
a different window giving a user control of many windows through
the menus.
.PP
A user might use this program as a datebook where each label has
the date and the command pops up a text window with the day's events.
.PP
This may be useful to keep a running history of user commands.
.PP
This may be used as a scratchpad of commands.
.PP
The user might keep a menu in each directory and popup the menu
whenever a 'cd' is executed.
.PP
.SH "BUGS"
When using 'olwm' (open look window manager) the XtPopup doesn't
seem to work properly.
.PP
If a >select_item is executed too many times, too fast,
the selection data used is garbage.
.PP
If a popup window is opened (!modify, !insert, or }text) and the
Top Level window size is changed (due to deleting or adding an item),
each popup window's size is changed.
.PP
After a menu is read twice (!read_menu) the keyboard translations
mysteriously disappear.
.PP
.SH "DIAGNOSTICS"
.\" Input String Too Large...Clearing
Only 1 Modify may take place at a time.
.br
.I "No more than 1 '!modify' may take place at a time."
.PP
Window Not Selected.  Command '!select' is needed first.
.br
.I "A '>select_command' was executed before a target window was selected."
.PP
Select Window Destroyed.  Please Select a New Window.
.br
.I "The target window was destroyed so a new one must be chosen."
.PP
Filename must not be blank.
.br
.I "A Read or Write operation needs a non-blank filename."
.PP
File '<fname>' Already Exists.
.br
.I "A Write operation was tried to an already existing file."
.PP
File '<fname>' Not Found.
.br
.I "A Read operation was desired but the file was not found."
.PP
Blank Menu File '<fname>'.
.br
.I "A Read operation was tried on a blank menu file."
.PP
ERROR: Failed to Become Selection Owner.
.br
.I "The menu could not become the selection owner."
.br
.I "No execution of any Select Commands to the target window can occur."
.PP
Command '<cmd>' currently in progress.
.br
.I "The multiple command menu item <cmd> must be completed first."
.PP
\'<menu_command>' Not Valid.  Commands: !exit,!raise,!lower,!select,!delete,!modify,!iconify.
.br
.I "The !menu_command is not valid. (See section Menu Commands)"
.\" Send Events MUST BE Allowed on the window for proper data transfer.
.\" A Menu Button was not chosen.
.\" A Proper Window was NOT Selected.
.PP
You may not delete a switch while it's being modified.
.br
.I "A switch may not be destroyed while a '!modify' is in progress."
.\" No selection or selection timed out.
.PP
Only 1 Insert may take place at a time.
.br
.I "No more than 1 '!insert' may take place at a time."
.PP
No more room in menu to add.
.br
.I "The maximum number of menu items has been reached."
.PP
Allocation error setting environment variable '<var>' to '<val>'
.br
.I "Memory could not be properly allocated for an environment variable."
.PP
Only 1 Setenv may take place at a time.
.br
.I "No more than 1 '!setenv' may take place concurrently."
.PP
.SH "FILES"
.I "/usr/lib/X11/app-defaults/XtMenu,"
.I "$HOME/.Xdefaults,"
and
.I "$HOME/XtMenu."
.PP
.SH "SEE ALSO"
tvtwm(1), X(1), xterm(1)
.PP
.SH "THANKS"
Iconifying a Window (comp.windows.x newsgroup, Frequently Asked Questions, Subject 111)
.br
Tom LaStrange for QueryPointer coordinate translation to
popup new menus in their correct locations.
.br
Bob Kierski for asciiTextWidgetClass help.
.br
Chris Roussi for ideas and discussion on implementation for
setting environment variables.  Also the desire for multiple
commands per menu item.
.br
Jan Rekers for input and great suggestions regarding fixes.
.PP
.SH "AUTHOR"
.PP
William Taylor (wrt@cray.com)
.br
Comments and Suggestions are Welcome.
